package com.lemoulinstudio.jnag.rds;

import java.io.Serializable;

/**
 * A class which represents the Id of an object in the Jnag system.
 *
 * @author Vincent Cantin
 */
class Id implements Serializable {

  private static final long serialVersionUID = 1L;

  private int value;

  /**
   * Builds an Id instance.
   * 
   * @param isLocal Is true if and only if this Id represents an object on the local host.
   *
   * @param isShared Is true if and only if this Id is binds to the same object on all the jnag sessions of this host.
   * It is only valid if <code>isLocal</code> is true, should be set to false otherwise.
   *
   * @param value The id number for this Id instance.
   */
  public Id(int value) {
    this.value = value;
  }

  public int getValue() {
    return value;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj instanceof Id) return (this.value == ((Id) obj).value);
    else return false;
  }

  @Override
  public int hashCode() {
    return value;
  }

  @Override
  public String toString() {
    return "[value=" + value + "]";
  }

}
