package com.lemoulinstudio.jnag.rds;

import com.sun.sgs.app.AppContext;
import com.sun.sgs.app.DataManager;
import com.sun.sgs.app.ManagedReference;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author Vincent Cantin
 */
class IdObjectBindingImpl<T> implements IdObjectBinding<T>, Serializable {

  private static final long serialVersionUID = 1L;

  private Map<Id, ManagedReference<? extends T>> idToObjectRef;
  private Map<ManagedReference<? extends T>, Id> objectRefToId;

  public IdObjectBindingImpl() {
    idToObjectRef = new HashMap<Id, ManagedReference<? extends T>>();
    objectRefToId = new HashMap<ManagedReference<? extends T>, Id>();
  }
  
  @Override
  public void bind(Id id, ManagedReference<? extends T> objectRef) {
    assert id != null : "Id should not be null.";
    assert !idToObjectRef.containsKey(id) : "The id is already used.";
    assert !objectRefToId.containsKey(objectRef) : "The object is already bound.";
    idToObjectRef.put(id, objectRef);
    objectRefToId.put(objectRef, id);
  }
  
  @Override
  public void unbindFromId(Id id) {
    assert idToObjectRef.containsKey(id) : "Nothing is bound to this id.";
    ManagedReference<? extends T> objectRef = idToObjectRef.remove(id);
    objectRefToId.remove(objectRef);
  }

  @Override
  public Id unbindFromObjectRef(ManagedReference<? extends T> objectRef) {
    assert objectRefToId.containsKey(objectRef) : "This object is not bound.";
    Id id = objectRefToId.remove(objectRef);
    idToObjectRef.remove(id);
    return id;
  }

  @Override
  public boolean isUsing(Id id) {
    return idToObjectRef.containsKey(id);
  }

  @Override
  public ManagedReference<? extends T> getObjectRef(Id id) {
    return idToObjectRef.get(id);
  }

  @Override
  public Id getId(ManagedReference<? extends T> objectRef) {
    return objectRefToId.get(objectRef);
  }

  @Override
  public void removingObject() {
    DataManager dataManager = AppContext.getDataManager();
    for (ManagedReference<? extends T> objectRef : objectRefToId.keySet())
      dataManager.removeObject(objectRef.get());
  }

}
