package com.lemoulinstudio.jnag.rds;

import com.sun.sgs.app.AppContext;
import com.sun.sgs.app.ManagedReference;
import com.sun.sgs.app.util.ScalableHashMap;
import java.io.Serializable;
import java.util.logging.Level;
import java.util.logging.Logger;
import com.lemoulinstudio.jnag.common.Remote;
import java.lang.reflect.Constructor;
import java.util.Map;

/**
 *
 * @author Vincent Cantin
 */
class ProxyFactoryImpl implements ProxyFactory, Serializable {

  private static final long serialVersionUID = 1L;

  private static String remoteClassNameToProxyClassBindingName = "JnagRemoteClassNameToProxyClass";

  static void initializeStaticObjects(AbstractConfiguration configuration) {
    ScalableHashMap<String, Class> remoteClassNameToProxyClass = new ScalableHashMap<String, Class>();

    for (Map.Entry<Class<? extends Remote>, Class> entry : configuration.getRemoteClassToProxyClass().entrySet())
      remoteClassNameToProxyClass.put(entry.getKey().getName(), entry.getValue());

    AppContext.getDataManager().setBinding(remoteClassNameToProxyClassBindingName, remoteClassNameToProxyClass);
  }

  private ManagedReference<ScalableHashMap<String, Class>> remoteClassNameToProxyClassRef;

  public ProxyFactoryImpl() {
    remoteClassNameToProxyClassRef = AppContext.getDataManager().createReference(
            (ScalableHashMap<String, Class>)
            AppContext.getDataManager().getBinding(remoteClassNameToProxyClassBindingName));
  }

  @Override
  public <T extends Remote> T createProxy(JnagSessionImpl jnagSession, Class<T> remoteClass) {
    try {
      Class<? extends Remote> proxyClass = remoteClassNameToProxyClassRef.get().get(remoteClass.getName());
      Constructor<? extends Remote> constructor = proxyClass.getConstructor(JnagSessionImpl.class);
      Remote proxy = constructor.newInstance(jnagSession);
      return (T) proxy;
    } catch (Exception ex) {
      Logger.getLogger(ProxyFactoryImpl.class.getName()).log(Level.SEVERE, null, ex);
      return null;
    }
  }
  
}
