package com.lemoulinstudio.jnag.rds;

import com.lemoulinstudio.jnag.common.BindToSharedId;
import com.sun.sgs.app.AppContext;
import com.sun.sgs.app.ManagedReference;
import com.sun.sgs.app.util.ScalableHashMap;
import java.io.Serializable;
import java.util.Map;

/**
 *
 * @author Vincent Cantin
 */
class SharedIdObjectBinding implements IdObjectBinding<BindToSharedId>, Serializable {

  private static final long serialVersionUID = 1L;

  private static String idToObjectRefBindingName = "JnagIdToObjectRef";
  private static String objectRefToIdBindingName = "JnagObjectRefToId";
  
  static void initializeStaticObjects() {
    AppContext.getDataManager().setBinding(idToObjectRefBindingName, new ScalableHashMap<Id, ManagedReference<? extends BindToSharedId>>());
    AppContext.getDataManager().setBinding(objectRefToIdBindingName, new ScalableHashMap<ManagedReference<? extends BindToSharedId>, Id>());
  }

  private static ManagedReference<Map<Id, ManagedReference<? extends BindToSharedId>>> getIdToObjectRef() {
    return AppContext.getDataManager().createReference((Map<Id, ManagedReference<? extends BindToSharedId>>)
            AppContext.getDataManager().getBinding(idToObjectRefBindingName));
  }

  private static ManagedReference<Map<ManagedReference<? extends BindToSharedId>, Id>> getObjectRefToId() {
    return AppContext.getDataManager().createReference((Map<ManagedReference<? extends BindToSharedId>, Id>)
            AppContext.getDataManager().getBinding(objectRefToIdBindingName));
  }

  private ManagedReference<Map<Id, ManagedReference<? extends BindToSharedId>>> idToObjectRef;
  private ManagedReference<Map<ManagedReference<? extends BindToSharedId>, Id>> objectRefToId;

  public SharedIdObjectBinding() {
    idToObjectRef = getIdToObjectRef();
    objectRefToId = getObjectRefToId();
  }

  @Override
  public void bind(Id id, ManagedReference<? extends BindToSharedId> objectRef) {
    assert id != null : "Id should not be null.";
    assert !idToObjectRef.get().containsKey(id) : "The id is already used.";
    assert !objectRefToId.get().containsKey(objectRef) : "The object is already bound.";
    idToObjectRef.getForUpdate().put(id, objectRef);
    objectRefToId.getForUpdate().put(objectRef, id);
  }

  @Override
  public void unbindFromId(Id id) {
    assert idToObjectRef.get().containsKey(id) : "Nothing is bound to this id.";
    ManagedReference<? extends BindToSharedId> objectRef = idToObjectRef.getForUpdate().remove(id);
    objectRefToId.getForUpdate().remove(objectRef);
  }

  @Override
  public Id unbindFromObjectRef(ManagedReference<? extends BindToSharedId> objectRef) {
    assert objectRefToId.get().containsKey(objectRef) : "This object is not bound.";
    Id id = objectRefToId.getForUpdate().remove(objectRef);
    idToObjectRef.getForUpdate().remove(id);
    return id;
  }

  @Override
  public boolean isUsing(Id id) {
    return idToObjectRef.get().containsKey(id);
  }

  @Override
  public ManagedReference<? extends BindToSharedId> getObjectRef(Id id) {
    return idToObjectRef.get().get(id);
  }

  @Override
  public Id getId(ManagedReference<? extends BindToSharedId> objectRef) {
    return objectRefToId.get().get(objectRef);
  }

  @Override
  public void removingObject() {
    throw new UnsupportedOperationException("Not supported.");
  }

}
